package es.rediris.papi.jaas;

import java.io.IOException;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import java.util.Map.Entry;

import javax.security.auth.Subject;
import javax.security.auth.callback.Callback;
import javax.security.auth.callback.CallbackHandler;
import javax.security.auth.callback.UnsupportedCallbackException;
import javax.security.auth.login.LoginException;
import javax.security.auth.spi.LoginModule;

import es.rediris.papi.PAPIPoA;

import org.apache.log4j.Logger;

public class PAPILoginModule implements LoginModule {

	
	private Subject subject;
    private CallbackHandler callbackHandler;
    private Map<String,?> sharedState;
    private Map<String,?> options;
    private PAPIPoA poa;
    private boolean authN;
    private boolean commited;
    private Map<String,String> userInfo;

	
	public PAPILoginModule() {
		super();
		this.poa = new PAPIPoA();
		// TODO Auto-generated constructor stub
	}

	public void initialize(Subject arg0, CallbackHandler arg1,
			Map<String, ?> arg2, Map<String, ?> arg3) {
		// TODO Auto-generated method stub
		this.subject = arg0;
		this.callbackHandler = arg1;
		this.sharedState = arg2;
		this.options = arg3;
		this.authN = false;
		this.commited = false;
		
		try {
			this.poa.init((String)this.options.get("configFile"));
		} catch (Exception e) {
			e.printStackTrace();
		}
		
		PAPIPoA.log.info("PAPILoginModule succesfully initialized");
		
	}

	public boolean login() throws LoginException {
		
		try {
			
			Callback[] callbacks = new Callback[2];
			callbacks[0] = new RequestCallback();
			callbacks[1] = new ResponseCallback();
			
		
			this.callbackHandler.handle(callbacks);
			
			if ( (((RequestCallback)callbacks[0]).getRequest() != null) && (((ResponseCallback)callbacks[1]).getResponse() != null) ) {
				this.userInfo = poa.doFilter(((RequestCallback)callbacks[0]).getRequest(), ((ResponseCallback)callbacks[1]).getResponse());
			}
			
			
			// Y si el doFilter devuelve cookies(authN=true)/NULL(authN=false)? => privateCredentials?????
			// Luego, descifrando las cookies a travs de "Ciphers", creamos subjects???
			//this.userInfo = ((RequestCallback)callbacks[0]).getRequest().getParameterMap();
			
			
		} catch (IOException e) {
			e.printStackTrace();
		} catch (UnsupportedCallbackException e) {
			e.printStackTrace();
		}
		
		this.authN = (this.userInfo != null);
		PAPIPoA.log.info("User login procedure finished with status: " + String.valueOf(this.authN));
		return this.authN;
	}

	public boolean commit() throws LoginException {
		
		if (!this.authN)
			return false;
		
		Set<Entry<String,String>> entries = this.userInfo.entrySet();
		Iterator<Entry<String,String>> it = entries.iterator();
		while (it.hasNext()) {
			Entry<String,String> entry = (Entry<String,String>)it.next();
			String key = entry.getKey();
			String value = entry.getValue();
			PAPIPrincipal p = new PAPIPrincipal(key);
			String separator = this.poa.getOption("PoAconf.valSeparator");
			p.setValues(value.split(separator));
			this.subject.getPrincipals().add(p);
			PAPIPoA.log.info("Added principal: " + p);
		}
		
		this.userInfo.clear();
		this.commited = true;
		
		return this.commited;
	}

	public boolean abort() throws LoginException {
		this.userInfo.clear();
		if (this.authN) {
			this.authN = false;
			return true;
		}
		
		return false;
	}

	public boolean logout() throws LoginException {
		Iterator it = this.subject.getPrincipals(PAPIPrincipal.class).iterator();
		while (it.hasNext()) {
			PAPIPrincipal p = (PAPIPrincipal)it.next();
			this.subject.getPrincipals().remove(p);
			PAPIPoA.log.info("Removed principal: " + p);
		}
		
		return true;
	}

}
